<?php
/*============================================================================ 
Title:		Download a file
Version:	1.1
Descript.:	This script provides a secure way to download a file.
Last Mod.:	19.12.2011
Author:		Andi Petzoldt
Email:		andi@petzoldt.net
Notes:		This script is written for the use with "CMS made simple"
Licence:	This program is free software; you can redistribute it and/or modify
			it under the terms of the GNU General Public License as published by
			the Free Software Foundation; either version 2 of the License, or
			(at your option) any later version.
			However, as a special exception to the GPL, this software is distributed
			as an addon module to CMS Made Simple.  You may not use this software
			in any Non GPL version of CMS Made simple, or in any version of CMS
			Made Simple that does not indicate clearly and obviously in its admin 
			section that the site was built with CMS Made Simple.
			Find more information about CMS Made Simple on its website:
			http://www.cmsmadesimple.org/
			This program is distributed in the hope that it will be useful,
			but WITHOUT ANY WARRANTY; without even the implied warranty of
			MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
			GNU General Public License for more details.
			You should have received a copy of the GNU General Public License
			along with this program; if not, write to the Free Software
			Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
			Or read it online: http://www.gnu.org/licenses/licenses.html#GPL
============================================================================*/


// include config
include('../config.php');

// Check GET data
$lang = 'en';
if (isset($_GET['l'])) $lang = $_GET['l']=='de' ? 'de' : 'en';
$file = '';
if (isset($_GET['f'])) { $file = urldecode($_GET['f']); } else { echo $lang=='de' ? 'Keine Datei angegeben' : 'No file specified'; exit; }
$pw = '';
if (isset($_GET['p'])) $pw = urldecode($_GET['p']);
$force = false;
if (isset($_GET['d'])) $force = $_GET['d']=='force' ? true : false;
$counter = false;
if (isset($_GET['c'])) $counter = (int)$_GET['c']>0 ? true : false;

// Check referer
$referer = isset($_SERVER['HTTP_REFERER']) ? $_SERVER['HTTP_REFERER'] : '';
if ($config['root_url']!=substr($referer,0,strlen($config['root_url']))) {
	header('HTTP/1.1 403 Forbidden');
	echo $lang=='de' ? 'Zugriff verweigert' : 'Access denied';
	exit;
}

// Check ending
$ending = true;
$ending = preg_match('/^(.*)\.(.*?)$/',$file,$tmp) ? strtolower($tmp[2]) : false;
if ($ending=='php') $force = false;

// Check and build path
$path = $config['uploads_path'].DIRECTORY_SEPARATOR.cleanPath($file);
if (!is_file($path) || !$ending) {
	header('HTTP/1.0 404 Not Found');
	if ($referer) header('Refresh:3;url='.$referer);
	echo $lang=='de' ? 'Datei nicht gefunden' : 'File not found';
	exit;
}

// Get filename
$temp = explode('/',$file);
$filename = str_replace('"','',array_pop($temp));

// Get blank path
$dir = $temp ? $config['uploads_path'].DIRECTORY_SEPARATOR.implode('/',$temp).DIRECTORY_SEPARATOR : $config['uploads_path'].DIRECTORY_SEPARATOR;

// Check password
if ($pw && is_file($dir.'.htpasswd')) {
	$check = substr(file_get_contents($dir.'.htpasswd'),9);
	if (crypt($pw,$check)!=$check) {
		header('HTTP/1.1 403 Forbidden');
		if ($referer) header('Refresh:3;url='.$referer);
		echo $lang=='de' ? 'Zugriff verweigert' : 'Access denied';
		exit;
	}
// If .htpasswd doesn't exists send error
} else if ($pw) {
	header('HTTP/1.1 403 Forbidden');
	if ($referer) header('Refresh:3;url='.$referer);
	echo $lang=='de' ? 'Passwort-Datei nicht gefunden' : 'Password file not found';
	exit;
} else if (is_file($dir.'.htpasswd')) {
	header('HTTP/1.1 403 Forbidden');
	if ($referer) header('Refresh:3;url='.$referer);
	echo $lang=='de' ? 'Kein Passwort angegeben' : 'No password given';
	exit;
}

// Count access to the file
if ($counter) {
	// Read counter file
	$count = array();
	if (is_file($dir.'.counter')) {
		$counts = @file($dir.'.counter');
		foreach($counts as $linenumber=>$data) {
			// Zeile aufteilen
			$line = explode('|',trim($data));
			$count[$line[1]] = $line[0];
		}
	}
	// Increase counter for file
	if (isset($count[$filename])) { $count[$filename]++; } else { $count[$filename] = 1; }
	// Write counter file
	$newcounter = array();
	foreach ($count as $k=>$v) {
		$newcounter[] = $v.'|'.$k;
	}
	@file_put_contents($dir.'.counter', implode("\n",$newcounter));
}

// Refer to file
if (!$force) {
	header('Location:'.$config['uploads_url'].'/'.cleanPath($file,'/'));
	exit;	
}

// Send headers
if ($force) {
	header('Content-Type: application/octet-stream');
	header('Content-Disposition: attachment; filename="'.$filename.'"'); 
	header('Content-Transfer-Encoding: binary');
	// Send the file
	readfile($path);
}


/**
* Cleans an URL path.
* @param string $path The path to be clean.
* @param string $type Should the path be cleaned as URL or as file path? (url/path)
* @return string
*/
function cleanPath($path,$sep=DIRECTORY_SEPARATOR) {
	// Clean path
	if ($path && $sep!='/') $path = str_replace ('/',$sep,trim($path));
	#if ($path) $path = preg_replace('#[^0-9a-zA-ZäöüÄÖÜß_/\\\. -]#','',$path);
	if ($path) while (substr($path,0,1) == $sep) { $path = substr($path,1); }
	if ($path) while (substr($path,-1) == $sep) { $path = substr($path,0,-1); }
	if ($path) $path = preg_replace('/\.+/','.',$path);
	if ($path) $path = preg_replace('/\/\./','/',$path);
	// Return cleaned path
	return $path;
}


// EOF